## Simulate data
######################
## Loading Packages ##
######################
library(foreach)
library(polycor)
library(psych)
library(ltm)
library(scrime)
library(sensitivity)
library(cluster)
library(stats)
library(mlogit)
library(missForest)
library(snowfall)

##################################
## point biserial and extension ##  
##################################
## x : continuous variable ##
## y : nominal variable    ##
biserial.cor2 <- function (x, y, use = c("all.obs", "complete.obs"), level = 2)
{
  if (!is.numeric(x))
    stop("'x' must be a numeric variable.\n")
  y <- as.factor(y)
  stopifnot(length(levs <- levels(y)) >= 2)
  if (length(x) != length(y))
    stop("'x' and 'y' do not have the same length")
  use <- match.arg(use)
  if (use == "complete.obs") {
    cc.ind <- complete.cases(x, y)
    x <- x[cc.ind]
    y <- y[cc.ind]
  }
  ## Point Biserial ##
  if (length(levs) == 2) { 
    ind <- y == levs[level]
    diff.mu <- mean(x[ind]) - mean(x[!ind])
    prob <- mean(ind)
    diff.mu * sqrt(prob * (1 - prob))/sd(x)
    ## Point Biserial Extension ##
  } else if (length(levs) > 2) { 
    n.m <- table(y)
    n <- sum(n.m)
    x.bar <- mean(x)
    x.bar.m <- sapply(split(x, y), mean)
    s.sqr <-  sum((x - x.bar)^2/n) #(n-1)/n*var(y)
    s.m <- n.m*(x.bar.m-x.bar)/n
    n*sum(s.m^2/(n.m*s.sqr))
  }
}
pcc2 <- function (x, dist = FALSE, corrected = TRUE, version = 1) 
{
  minrc <- function(x){
    rx<-max(x,na.rm=TRUE)
    n.lev<-numeric(nrow(x))
    for(i in min(x,na.rm=TRUE):rx){
      tmp<-x==i
      tmp2<-rowSums(tmp,na.rm=TRUE)>0
      n.lev<-n.lev+tmp2
    }
    uni.lev<-sort(unique(n.lev),decreasing=TRUE)
    #if(uni.lev[1]!=rx)
    #        stop("No variable shows all possible levels.",call.=FALSE)
    if(length(uni.lev)==1)
      return(uni.lev)
    mat.rc<-matrix(rx,nrow(x),nrow(x))
    for(i in uni.lev[-1]){
      ids<-which(n.lev==i)
      mat.rc[ids,]<-i
      mat.rc[,ids]<-i
    }
    mat.rc
  }
  #if (!version %in% (1:3)) 
  #stop("version must be between 1 and 3.")
  if (dist & !corrected) 
    stop("corrected must be TRUE, if dist = TRUE.")
  chi2 <- scrime::rowChisqStats(x, compPval = FALSE)
  chi2 <- chi2 + t(chi2)
  if (any(is.na(x))) {
    naIdentifier <- !is.na(x)
    mat.n <- naIdentifier %*% t(naIdentifier)
  }
  else mat.n <- ncol(x)
  pcc <- chi2/(chi2 + mat.n)
  mat.rc <- minrc(x)
  mat.rc <- mat.rc/(mat.rc - 1)
  #print(mat.rc)
  #print(diag(pcc))
  diag(pcc) <- if (is.matrix(mat.rc)) 
    1/diag(mat.rc)
  else 1/mat.rc
  if (corrected) 
    pcc <- mat.rc * pcc
  colnames(pcc) <- rownames(pcc) <- rownames(x)
  if (!dist) {
    pcc <- sqrt(pcc)
    return(pcc)
  }
  pcc <- switch(version, sqrt(1 - pcc), 1 - sqrt(pcc), 1 - pcc)
  pcc
}

##############################################
## Function to establish Correlation Matrix ##                 
##############################################
calCor <- function(dat, types) {
  stopifnot(length(types)==2)
  
  if("con" %in% types) {
    if(all(types=="con")) {  ## con vs con: spearman's R
      res <- cor(as.numeric(dat[,1]), as.numeric(dat[,2]), use="pairwise.complete.obs",method="spearman")
    } else if("ord" %in% types) { ## con vs ord: polyserial
      dat <- na.omit(dat)
      ord.idx <- which(types=="ord")
      level <- nlevels(as.factor(dat[,ord.idx]))-(max(dat[,ord.idx])-min(dat[,ord.idx])+1)
      value <- nlevels(as.factor(dat[,ord.idx]))
      res <- NA
      if((level != 0)|(value<3)|(length(unique(dat[,setdiff(1:ncol(dat),ord.idx)]))<3)){res <- NA}
      else{
        res <- polycor::polyserial(as.numeric(dat[,setdiff(1:ncol(dat),ord.idx)]),as.numeric(dat[,ord.idx])) #polyserial
        if(abs(res)==1){
          n1 <- ord.idx
          ord <- as.factor(dat[,n1])
          dat[,n1] <- dat[,n1] - min(dat[,n1]) + 1
          dat.2 <- dat[,n1]
          n_old <- 0
          n <- vector(length=nlevels(ord))
          s <- min(dat[,n1])
          for (i in s:(nlevels(ord)+s-1)){
            no <- which(dat.2==i)
            n[i+1-s] <- length(no)
            new.level.1 <- 0.5*(1+n_old+n[i+1-s]+n_old)
            n_old <- sum(n[1:(i+1-s)])  
            dat[no,n1] <- new.level.1
          }    
          res <- cor(as.numeric(dat[,1]), as.numeric(dat[,2]), use="pairwise.complete.obs",method="pearson")}}
    } else if("nom" %in% types) { ## con vs nom: point biserial and extension
      dat <- na.omit(dat)
      level <- nlevels(as.factor(dat[,which(types%in%c("nom"))]))-(max(dat[,which(types%in%c("nom"))])-min(dat[,which(types%in%c("nom"))])+1)
      value <- nlevels(as.factor(dat[,which(types%in%c("nom"))]))
      if((level != 0)|(value<2)|(length(which(dat[,which(types%in%c("nom"))]==1))<2)|(length(unique(dat[,which(types%in%c("con"))]))<3)){res <- NA}
      else{
        nom.idx <- which(types=="nom")
        res <- biserial.cor2(as.numeric(dat[,setdiff(1:ncol(dat),nom.idx)]),as.numeric(dat[,nom.idx]), use="complete.obs") #biserial or multivariate
      } }}
  else if("ord" %in% types) {
    if(all(types=="ord")) { ## ord vs ord: polychoric correlation and pearson's r 
      dat <- na.omit(dat)  
      n1 <- as.factor(dat[,1])
      n2 <- as.factor(dat[,2])
      level.1 <- nlevels(n1)-(max(dat[,1])-min(dat[,1])+1)
      value.1 <- nlevels(n1)  
      level.2 <- nlevels(n2)-(max(dat[,2])-min(dat[,2])+1)
      value.2 <- nlevels(n2)
      if((level.2 != 0)|(value.2<3)|(level.1 != 0)|(value.1<3)){res <- NA}
      else if(min(table(dat[,1],dat[,2]))<3) {
        n_old <- 0
        n <- vector(length=nlevels(n2))
        s <- min(dat[,2])
        for (i in s:(nlevels(n2)+s-1)){
          no <- which(dat[,2]==i)
          n[i+1-s] <- length(no)
          new.level.2 <- 0.5*(1+n_old+n[i+1-s]+n_old)
          n_old <- sum(n[i+1-s:i])  
          dat[no,2] <- rep(new.level.2,length(no))
        }
        res <- cor(as.numeric(dat[,1]), as.numeric(dat[,2]), use="pairwise.complete.obs",method="pearson")}
      else{
        res <- polychor(dat[,1],dat[,2])
      }
    }
    else if("nom" %in% types) { #ord vs nom:rank biserial: point biserial and extension
      dat <- na.omit(dat)
      n1 <- as.factor(dat[,1])
      n2 <- as.factor(dat[,2])
      ord.idx <- which(types=="ord")
      nom.idx <- which(types=="nom")
      level.1 <- nlevels(as.factor(dat[,ord.idx]))-(max(dat[,ord.idx])-min(dat[,ord.idx])+1)
      value.1 <- nlevels(as.factor(dat[,ord.idx]))  
      level.2 <- nlevels(as.factor(dat[,nom.idx]))-(max(dat[,nom.idx])-min(dat[,nom.idx])+1)
      value.2 <- nlevels(as.factor(dat[,nom.idx]))
      if((level.2 != 0)|(value.1<3)|(level.1 != 0)|(value.2<2)|(length(which(dat[,which(types%in%c("nom"))]==1))<2)){res <- NA}
      else{
        n1 <- ord.idx
        ord <- as.factor(dat[,n1])
        dat[,n1] <- dat[,n1] - min(dat[,n1]) + 1
        dat.2 <- dat[,n1]
        n_old <- 0
        n <- vector(length=nlevels(ord))
        s <- min(dat[,n1])
        for (i in s:(nlevels(ord)+s-1)){
          no <- which(dat.2==i)
          n[i+1-s] <- length(no)
          new.level.1 <- 0.5*(1+n_old+n[i+1-s]+n_old)
          n_old <- sum(n[1:(i+1-s)])  
          dat[no,n1] <- new.level.1
        }
        dat[,1] <- factor(dat[,1]); dat[,2] <- factor(dat[,2])
        res <- biserial.cor2(as.numeric(dat[,setdiff(1:ncol(dat),ord.idx)]), dat[,ord.idx],use="complete.obs")} 
      #biserial or multivariat
    }
  }   
  else { ## nom vs nom: phi pearson's C and cramer's V
    stopifnot(all(types=="nom"))
    dat <- na.omit(dat)
    n1 <- as.factor(dat[,1]); n2 <- as.factor(dat[,2])
    level.1 <- nlevels(as.factor(dat[,1]))-(max(dat[,1])-min(dat[,1])+1)
    value.1 <- nlevels(as.factor(dat[,1]))  
    level.2 <- nlevels(as.factor(dat[,2]))-(max(dat[,2])-min(dat[,2])+1)
    value.2 <- nlevels(as.factor(dat[,2]))
    if((level.2 != 0)|(value.2<2)|(level.1 != 0)|(value.1<2)|(length(which(dat[,1]==1))<2)|(length(which(dat[,2]==1))<2)){res <- NA}
    else{
      if((nlevels(n1)==2 & nlevels(n2)==2)== TRUE) {
        dat <- as.data.frame(dat)
        res <- psych::phi(table(dat), digits=5)
      } 
      else if(nlevels(n1)!=nlevels(n2)){
        L <- min(nlevels(n1),nlevels(n2))
        dat[,1] <- dat[,1] - min(dat[,1]) + 1
        dat[,2] <- dat[,2] - min(dat[,2]) + 1
        chi.s <- scrime::rowChisqStats(t(as.matrix(dat)),compPval = FALSE)
        chi.s <- chi.s +t(chi.s)
        res <- (chi.s[2,1]/(nrow(dat)*ncol(dat)*(L-1)))^0.5}
      else 
      {dat <- as.data.frame(dat)
       res <- pcc2(t(dat))[2,1]
      }}
  }
  res
}

###################################
## Calculate the Distance matrix ##
###################################
## revise this function on 4/23/2013
mixedDist <- function(dat, types, isAbs=TRUE) {
  stopifnot(levels(factor(types))%in%c("con","ord","nom"))
  stopifnot(ncol(dat)==length(types))
  dist <- matrix(0,ncol(dat),ncol(dat))
  pairwise.cor = lapply(as.list(c(1:(ncol(dat)-1))),function(x) sapply(c((x+1):ncol(dat)),function(y) calCor(dat[,c(x,y)], types[c(x,y)])))
  for (i in 1:(ncol(dat)-1)) {
    dist[i,(i+1):ncol(dat)] = 1 - abs(pairwise.cor[[i]])
  }
  return(dist)
}

simulate.data <- function(sub.no,var.no,no.var.group,no.sub.group,var.per.group,sub.per.group){
  ## sub.no: total number of subjects
  ## var.no: total number of variables
  ## no.var.group: number of variables group that are correlated
  ## no.sub.group: number of subjects group that are correlated
  ## first generate correlated variables
  set.seed(12345)
  Var.assignment = rpois(no.var.group,var.per.group)
  Sub.assignment = rpois(no.sub.group,sub.per.group)
  
  data <- matrix(0,sub.no,var.no)
  basis <- matrix(0,sub.no,no.var.group)
  coefficient.each.group <- vector("list",length=no.var.group)
  for (i in 1:no.var.group){
    basis[,i] <- rnorm(sub.no,sample((-2:2),1),4)
    coefficient.each.group[[i]] <- matrix(rnorm(Var.assignment[i]*2,0,1),Var.assignment[i],2)
    if(i==1){var=0}
    else{var=sum(Var.assignment[1:(i-1)])}
    for (j in 1:Var.assignment[i]){
      data[,j+var] <- basis[,i]*coefficient.each.group[[i]][j,1] + coefficient.each.group[[i]][j,2]
    }
  }
  var <- sum(Var.assignment)
  for (i in (var+1):var.no){
    data[,i] <- rnorm(sub.no,sample(0,1),4)
  }
  a = 1-abs(cor(data))
  heatmap(a)
  data.all <- simulate.data.sub(data,Sub.assignment)
  ## generate outliers concentrate in subject 10,50,100,150,200,250,300,350,400,450,500,550,600
  ##                                  variable 20,40,60,80,100,120,140,160,180,200,220,240,260
  list(list(data.all,Var.assignment,Sub.assignment))
}


## generate subjects correlated part, based on d
simulate.data.sub <- function(d,Sub.assignment){
  for (i in 1:length(Sub.assignment)){
    if(i==1){sub=0}
    else{sub=sum(Sub.assignment[1:(i-1)])}
    basis <- rnorm(300,1,2)
    for (j in 1:Sub.assignment[i]){
      d[j+sub,] <- basis + d[j+sub,]
    }
  }
  sub = sum(Sub.assignment)
  for( i in (sub+1):600){
    d[i,] = d[i,] + rnorm(300,0,4)
  }
  return(d)
}


convert.var.type <- function(Data,nom,ord){
  convert.ord <- function(x){
    n.level <- sample(2:5,1)
    cut = sample(sort(unique(x))[-c(1,length(unique(x)))],n.level)
    cut.sort = sort(c(min(x)-0.1,cut,max(x)+0.1))
    factor.var = cut(x,cut.sort)
    levels(factor.var) = c(1:(n.level+1))
    out = as.numeric(factor.var)
    return(out)
  }
  convert.nom <- function(x){
    n.level <- sample(2:5,1)
    cut = sample(sort(unique(x))[-c(1,length(unique(x)))],n.level)
    cut.sort = sort(c(min(x)-0.1,cut,max(x)+0.1))
    factor.var = cut(x,cut.sort)
    levels(factor.var) = sample(c(1:(n.level+1)),n.level+1,replace=FALSE)
    out = as.numeric(factor.var)
    return(out)
  }
  Data.new = Data
  Data.new[,nom] = apply(Data[,nom],2,convert.nom)
  Data.new.new = Data.new
  Data.new.new[,ord] = apply(Data.new[,ord],2,convert.ord)
  return(Data.new.new)
}
############################################
## Simulation 1: 6 clsuter Var and Sub######
############################################
set.seed(12345)
out.1 <- simulate.data(600,300,6,6,40,80)

nom = sample(1:300,100)
ord = sample(c(1:300)[-nom],60)
Simulate.Mixed.data.1 = convert.var.type(out.1[[1]][[1]],nom,ord)
Type = rep("con",300)
Type[nom] = "nom"
Type[ord] = "ord"
Data.1 = list(Simulate.Mixed.data.1,Type)
row.names(Data.1[[1]]) = paste("Sub",1:nrow(Data.1[[1]]),sep=".")
colnames(Data.1[[1]]) = paste("Var",1:ncol(Data.1[[1]]),sep=".")

save(Data.1,file="Data.1.rdata")

Mixed.data.1 = mixedDist(Simulate.Mixed.data.1,Type)
Mixed.data.1.var = Mixed.data.1+t(Mixed.data.1)
Simulate.Mixed.data.1.reform = Simulate.Mixed.data.1
for(i in 1:length(which(Type=="con"))){
  Simulate.Mixed.data.1.reform[,which(Type=="con")[i]] <- as.numeric(Simulate.Mixed.data.1[,which(Type=="con")[i]])}
for(i in 1:length(which(Type=="ord"))){
  Simulate.Mixed.data.1.reform[,which(Type=="ord")[i]] <- as.ordered(Simulate.Mixed.data.1[,which(Type=="ord")[i]])}
for(i in 1:length(which(Type=="nom"))){
  Simulate.Mixed.data.1.reform[,which(Type=="nom")[i]] <- as.factor(Simulate.Mixed.data.1[,which(Type=="nom")[i]])}

Mixed.data.1.sub <- as.matrix(daisy(Simulate.Mixed.data.1,metric="gower"))
var.color = Type
var.color[which(Type=="con")] = "blue"
var.color[which(Type=="ord")] = "yellow"
var.color[which(Type=="nom")] = "red"

png("Mixed.Var.Distance.Simulation.1.png",width=700,height=700)
par(cex.main=3,mar=c(1,10,5,5))
heatmap(1-Mixed.data.1.var,col= grey(seq(1,0,-0.001)),Rowv =NA,Colv =NA,main="(a)")
dev.off()

png("Mixed.Sub.Distance.Simulation.1.png",width=700,height=700)
par(cex.main=3,mar=c(1,10,5,5))
## heatmap.2(Mixed.data.1.sub,col= greenred(256),Rowv =FALSE,Colv =FALSE, scale="none", 
##          trace="none", cexRow=0.8, cexCol=0.125,margins=c(4,14), symm=FALSE, symkey=TRUE,main="Heatmap of Subject Distance \n(Simulation1)")
heatmap(1-Mixed.data.1.sub,col= grey(seq(1,0,-0.001)),Rowv =NA,Colv =NA,main="(b)")
dev.off()

#############################################
## Simulation 2: 20 cluster Var and Sub######
#############################################
set.seed(12345)
out.1 <- simulate.data(600,300,20,20,15,25)

nom = sample(1:300,100)
ord = sample(c(1:300)[-nom],60)
Simulate.Mixed.data.1 = convert.var.type(out.1[[1]][[1]],nom,ord)
Type = rep("con",300)
Type[nom] = "nom"
Type[ord] = "ord"
Data.2 = list(Simulate.Mixed.data.1 ,Type)
row.names(Data.2[[1]]) = paste("Sub",1:nrow(Data.2[[1]]),sep=".")
colnames(Data.2[[1]]) = paste("Var",1:ncol(Data.2[[1]]),sep=".")

save(Data.2,file="Data.2.rdata")


###################################################
## Simulation 3: no Var cluster and 40 Sub clusters
###################################################
## only subject correlation
no.sub.group=40
sub.per.group=14
set.seed(12345)
Sub.assignment = rpois(no.sub.group,sub.per.group)
sub.no=600
var.no=300
## 200 variable same
range.group = c(0,cumsum(Sub.assignment))
data <- matrix(rnorm(sub.no*var.no,0,4),sub.no,var.no)
for (i in 1:no.sub.group){
  basis <- rnorm(300,0,4)
  # var.same = sample(1:300,200)
  for(j in c(range.group[i]+1):c(range.group[i+1])){
    data[j,] = basis + rnorm(300,0,0.01)
  }
}
basis <- rnorm(300,0,4)
for( i in range.group[no.sub.group+1]:sub.no){
  data[i,] = basis + rnorm(300,0,0.01)
}
nom = sample(c(1:300),300*0.65)
ord = sample(c(1:300)[-nom],30)
convert.var.type <- function(Data,nom,ord){
  convert.ord <- function(x){
    n.level <- sample(2:30,1)
    cut.1 = sample(c(seq(0.01,0.05,0.001),seq(0.95,0.99,0.001)),1)
    cut.2 =  sample(seq(0.01,0.99,0.001)[-which(abs(seq(0.01,0.99,0.001)-cut.1)<1e-10)],n.level-1)
    cut=sort(quantile(x,c(cut.1,cut.2)))
    cut.sort = sort(c(min(x)-0.1,cut,max(x)+0.1))
    factor.var = cut(x,cut.sort)
    levels(factor.var) = c(1:(n.level+1))
    out = as.numeric(factor.var)
    return(out)
  }
  convert.nom <- function(x){
    n.level <- sample(1:30,1)
    if(n.level==1){cut.1 = sample(seq(0.01,0.99,0.01),1)
                   cut=sort(quantile(x,cut.1))}
    else{
      cut.1 = sample(c(seq(0.01,0.05,0.001),seq(0.95,0.99,0.001)),1)
      cut.2 =  sample(seq(0.01,0.99,0.001)[-which(abs(seq(0.01,0.99,0.001)-cut.1)<1e-10)],n.level-1)
      cut=sort(quantile(x,c(cut.1,cut.2)))
    }
    cut.sort = sort(c(min(x)-0.1,cut,max(x)+0.1))
    factor.var = cut(x,cut.sort)
    levels(factor.var) = sample(c(1:(n.level+1)),n.level+1,replace=FALSE)
    out = as.numeric(factor.var)
    return(out)
  }
  Data.new = Data
  Data.new[,nom] = apply(Data[,nom],2,convert.nom)
  Data.new.new = Data.new
  Data.new.new[,ord] = apply(Data.new[,ord],2,convert.ord)
  return(Data.new.new)
}

Simulate.Mixed.data.1 = convert.var.type(data,nom,ord)
Type = rep("con",300)
Type[nom] = "nom"
Type[ord] = "ord"
Data.3 = list(Simulate.Mixed.data.1,Type)
row.names(Data.3[[1]]) = paste("Sub",1:nrow(Data.3[[1]]),sep=".")
colnames(Data.3[[1]]) = paste("Var",1:ncol(Data.3[[1]]),sep=".")
save(Data.3,file="Data.3.rdata")
