## Select K
######################
## Loading Packages ##
######################
library(foreach)
library(polycor)
library(psych)
library(ltm)
library(scrime)
library(sensitivity)
library(cluster)
library(stats)
library(mlogit)
library(snowfall)

##################################
## point biserial and extension ##  
##################################
## x : continuous variable ##
## y : nominal variable    ##
biserial.cor2 <- function (x, y, use = c("all.obs", "complete.obs"), level = 2)
{
  if (!is.numeric(x))
    stop("'x' must be a numeric variable.\n")
  y <- as.factor(y)
  stopifnot(length(levs <- levels(y)) >= 2)
  if (length(x) != length(y))
    stop("'x' and 'y' do not have the same length")
  use <- match.arg(use)
  if (use == "complete.obs") {
    cc.ind <- complete.cases(x, y)
    x <- x[cc.ind]
    y <- y[cc.ind]
  }
  ## Point Biserial ##
  if (length(levs) == 2) { 
    ind <- y == levs[level]
    diff.mu <- mean(x[ind]) - mean(x[!ind])
    prob <- mean(ind)
    diff.mu * sqrt(prob * (1 - prob))/sd(x)
    ## Point Biserial Extension ##
  } else if (length(levs) > 2) { 
    n.m <- table(y)
    n <- sum(n.m)
    x.bar <- mean(x)
    x.bar.m <- sapply(split(x, y), mean)
    s.sqr <-  sum((x - x.bar)^2/n) #(n-1)/n*var(y)
    s.m <- n.m*(x.bar.m-x.bar)/n
    n*sum(s.m^2/(n.m*s.sqr))
  }
}
pcc2 <- function (x, dist = FALSE, corrected = TRUE, version = 1) 
{
  minrc <- function(x){
    rx<-max(x,na.rm=TRUE)
    n.lev<-numeric(nrow(x))
    for(i in min(x,na.rm=TRUE):rx){
      tmp<-x==i
      tmp2<-rowSums(tmp,na.rm=TRUE)>0
      n.lev<-n.lev+tmp2
    }
    uni.lev<-sort(unique(n.lev),decreasing=TRUE)
    #if(uni.lev[1]!=rx)
    #        stop("No variable shows all possible levels.",call.=FALSE)
    if(length(uni.lev)==1)
      return(uni.lev)
    mat.rc<-matrix(rx,nrow(x),nrow(x))
    for(i in uni.lev[-1]){
      ids<-which(n.lev==i)
      mat.rc[ids,]<-i
      mat.rc[,ids]<-i
    }
    mat.rc
  }
  #if (!version %in% (1:3)) 
  #stop("version must be between 1 and 3.")
  if (dist & !corrected) 
    stop("corrected must be TRUE, if dist = TRUE.")
  chi2 <- scrime::rowChisqStats(x, compPval = FALSE)
  chi2 <- chi2 + t(chi2)
  if (any(is.na(x))) {
    naIdentifier <- !is.na(x)
    mat.n <- naIdentifier %*% t(naIdentifier)
  }
  else mat.n <- ncol(x)
  pcc <- chi2/(chi2 + mat.n)
  mat.rc <- minrc(x)
  mat.rc <- mat.rc/(mat.rc - 1)
  #print(mat.rc)
  #print(diag(pcc))
  diag(pcc) <- if (is.matrix(mat.rc)) 
    1/diag(mat.rc)
  else 1/mat.rc
  if (corrected) 
    pcc <- mat.rc * pcc
  colnames(pcc) <- rownames(pcc) <- rownames(x)
  if (!dist) {
    pcc <- sqrt(pcc)
    return(pcc)
  }
  pcc <- switch(version, sqrt(1 - pcc), 1 - sqrt(pcc), 1 - pcc)
  pcc
}

##############################################
## Function to establish Correlation Matrix ##                 
##############################################
calCor <- function(dat, types) {
  stopifnot(length(types)==2)
  
  if("con" %in% types) {
    if(all(types=="con")) {  ## con vs con: spearman's R
      res <- cor(as.numeric(dat[,1]), as.numeric(dat[,2]), use="pairwise.complete.obs",method="spearman")
    } else if("ord" %in% types) { ## con vs ord: polyserial
      dat <- na.omit(dat)
      ord.idx <- which(types=="ord")
      level <- nlevels(as.factor(dat[,ord.idx]))-(max(dat[,ord.idx])-min(dat[,ord.idx])+1)
      value <- nlevels(as.factor(dat[,ord.idx]))
      res <- NA
      if((level != 0)|(value<3)|(length(unique(dat[,setdiff(1:ncol(dat),ord.idx)]))<3)){res <- NA}
      else{
        res <- polycor::polyserial(as.numeric(dat[,setdiff(1:ncol(dat),ord.idx)]),as.numeric(dat[,ord.idx])) #polyserial
        if(abs(res)==1){
          n1 <- ord.idx
          ord <- as.factor(dat[,n1])
          dat[,n1] <- dat[,n1] - min(dat[,n1]) + 1
          dat.2 <- dat[,n1]
          n_old <- 0
          n <- vector(length=nlevels(ord))
          s <- min(dat[,n1])
          for (i in s:(nlevels(ord)+s-1)){
            no <- which(dat.2==i)
            n[i+1-s] <- length(no)
            new.level.1 <- 0.5*(1+n_old+n[i+1-s]+n_old)
            n_old <- sum(n[1:(i+1-s)])  
            dat[no,n1] <- new.level.1
          }    
          res <- cor(as.numeric(dat[,1]), as.numeric(dat[,2]), use="pairwise.complete.obs",method="pearson")}}
    } else if("nom" %in% types) { ## con vs nom: point biserial and extension
      dat <- na.omit(dat)
      level <- nlevels(as.factor(dat[,which(types%in%c("nom"))]))-(max(dat[,which(types%in%c("nom"))])-min(dat[,which(types%in%c("nom"))])+1)
      value <- nlevels(as.factor(dat[,which(types%in%c("nom"))]))
      if((level != 0)|(value<2)|(length(which(dat[,which(types%in%c("nom"))]==1))<2)|(length(unique(dat[,which(types%in%c("con"))]))<3)){res <- NA}
      else{
        nom.idx <- which(types=="nom")
        res <- biserial.cor2(as.numeric(dat[,setdiff(1:ncol(dat),nom.idx)]),as.numeric(dat[,nom.idx]), use="complete.obs") #biserial or multivariate
      } }}
  else if("ord" %in% types) {
    if(all(types=="ord")) { ## ord vs ord: polychoric correlation and pearson's r 
      dat <- na.omit(dat)  
      n1 <- as.factor(dat[,1])
      n2 <- as.factor(dat[,2])
      level.1 <- nlevels(n1)-(max(dat[,1])-min(dat[,1])+1)
      value.1 <- nlevels(n1)  
      level.2 <- nlevels(n2)-(max(dat[,2])-min(dat[,2])+1)
      value.2 <- nlevels(n2)
      if((level.2 != 0)|(value.2<3)|(level.1 != 0)|(value.1<3)){res <- NA}
      else if(min(table(dat[,1],dat[,2]))<3) {
        n_old <- 0
        n <- vector(length=nlevels(n2))
        s <- min(dat[,2])
        for (i in s:(nlevels(n2)+s-1)){
          no <- which(dat[,2]==i)
          n[i+1-s] <- length(no)
          new.level.2 <- 0.5*(1+n_old+n[i+1-s]+n_old)
          n_old <- sum(n[i+1-s:i])  
          dat[no,2] <- rep(new.level.2,length(no))
        }
        res <- cor(as.numeric(dat[,1]), as.numeric(dat[,2]), use="pairwise.complete.obs",method="pearson")}
      else{
        res <- polychor(dat[,1],dat[,2])
      }
    }
    else if("nom" %in% types) { #ord vs nom:rank biserial: point biserial and extension
      dat <- na.omit(dat)
      n1 <- as.factor(dat[,1])
      n2 <- as.factor(dat[,2])
      ord.idx <- which(types=="ord")
      nom.idx <- which(types=="nom")
      level.1 <- nlevels(as.factor(dat[,ord.idx]))-(max(dat[,ord.idx])-min(dat[,ord.idx])+1)
      value.1 <- nlevels(as.factor(dat[,ord.idx]))  
      level.2 <- nlevels(as.factor(dat[,nom.idx]))-(max(dat[,nom.idx])-min(dat[,nom.idx])+1)
      value.2 <- nlevels(as.factor(dat[,nom.idx]))
      if((level.2 != 0)|(value.1<3)|(level.1 != 0)|(value.2<2)|(length(which(dat[,which(types%in%c("nom"))]==1))<2)){res <- NA}
      else{
        n1 <- ord.idx
        ord <- as.factor(dat[,n1])
        dat[,n1] <- dat[,n1] - min(dat[,n1]) + 1
        dat.2 <- dat[,n1]
        n_old <- 0
        n <- vector(length=nlevels(ord))
        s <- min(dat[,n1])
        for (i in s:(nlevels(ord)+s-1)){
          no <- which(dat.2==i)
          n[i+1-s] <- length(no)
          new.level.1 <- 0.5*(1+n_old+n[i+1-s]+n_old)
          n_old <- sum(n[1:(i+1-s)])  
          dat[no,n1] <- new.level.1
        }
        dat[,1] <- factor(dat[,1]); dat[,2] <- factor(dat[,2])
        res <- biserial.cor2(as.numeric(dat[,setdiff(1:ncol(dat),ord.idx)]), dat[,ord.idx],use="complete.obs")} 
      #biserial or multivariat
    }
  }   
  else { ## nom vs nom: phi pearson's C and cramer's V
    stopifnot(all(types=="nom"))
    dat <- na.omit(dat)
    n1 <- as.factor(dat[,1]); n2 <- as.factor(dat[,2])
    level.1 <- nlevels(as.factor(dat[,1]))-(max(dat[,1])-min(dat[,1])+1)
    value.1 <- nlevels(as.factor(dat[,1]))  
    level.2 <- nlevels(as.factor(dat[,2]))-(max(dat[,2])-min(dat[,2])+1)
    value.2 <- nlevels(as.factor(dat[,2]))
    if((level.2 != 0)|(value.2<2)|(level.1 != 0)|(value.1<2)|(length(which(dat[,1]==1))<2)|(length(which(dat[,2]==1))<2)){res <- NA}
    else{
      if((nlevels(n1)==2 & nlevels(n2)==2)== TRUE) {
        dat <- as.data.frame(dat)
        res <- psych::phi(table(dat), digits=5)
      } 
      else if(nlevels(n1)!=nlevels(n2)){
        L <- min(nlevels(n1),nlevels(n2))
        dat[,1] <- dat[,1] - min(dat[,1]) + 1
        dat[,2] <- dat[,2] - min(dat[,2]) + 1
        chi.s <- scrime::rowChisqStats(t(as.matrix(dat)),compPval = FALSE)
        chi.s <- chi.s +t(chi.s)
        res <- (chi.s[2,1]/(nrow(dat)*ncol(dat)*(L-1)))^0.5}
      else 
      {dat <- as.data.frame(dat)
       res <- pcc2(t(dat))[2,1]
      }}
  }
  res
}

###################################
## Calculate the Distance matrix ##
###################################
## revise this function on 4/23/2013
mixedDist <- function(dat, types, isAbs=TRUE) {
  stopifnot(levels(factor(types))%in%c("con","ord","nom"))
  stopifnot(ncol(dat)==length(types))
  dist <- matrix(0,ncol(dat),ncol(dat))
  pairwise.cor = lapply(as.list(c(1:(ncol(dat)-1))),function(x) sapply(c((x+1):ncol(dat)),function(y) calCor(dat[,c(x,y)], types[c(x,y)])))
  for (i in 1:(ncol(dat)-1)) {
    dist[i,(i+1):ncol(dat)] = 1 - abs(pairwise.cor[[i]])
  }
  return(dist)
}

########################################
####### Generate missing value #########
########################################
## revise this function on 4/23/2013
missing.no <- function(data,n,perc){
  missing.data <- vector("list",length=n)
  m <- ncol(data)*nrow(data)
  set.seed(12345)   ## use this seed
  missing.data = lapply(1:n,function(x) sample(1:m,round(m*perc)))
  return(missing.data)
}

#################################################
####### Generate secondary missing value #########
#################################################
## revise this function on 4/23/2013

missing.no.2 <- function(data,n,perc,missing.1){
  missing.data <- vector("list",length=n)
  m <- ncol(data)*nrow(data)
  m.2 <- c(1:m)[-missing.1]
  total <- length(m.2)
  set.seed(12345)   ## use this seed
  missing.data = lapply(1:n,function(x) sample(m.2,round(total*perc)))
  return(missing.data)
}

missing.no.3 <- function(data,n,perc,missing.1,missing.2){
  missing.data <- vector("list",length=n)
  m <- ncol(data)*nrow(data)
  m.2 <- c(1:m)[-c(missing.1,missing.2)]
  total <- length(m.2)
  set.seed(12345)   ## use this seed
  missing.data = lapply(1:n,function(x) sample(m.2,round(total*perc)))
  return(missing.data)
}
#############################################
## Get final weighted imputed value: KNN.v ##
#############################################
## revise this function on 4/23/2013
determin.i <- function(k.imputed,weight,miss.type,w){
  ## weight could be adjusted according to user's preference
  if (miss.type%in%"con"){imputed.value <- weighted.mean(k.imputed,weight^w)}
  else if(miss.type%in%"nom"){ 
    p.number <- length(table(k.imputed))
    weight.sum <- tapply(weight,k.imputed,sum)
    if (length(max(weight.sum))>1){imputed.value <- sample(as.numeric(names(which.max(weight.sum))),1)}
    else{imputed.value <- as.numeric(names(which.max(weight.sum)))}
  } 
  else {imputed.value <- round(weighted.mean(k.imputed,weight^w))}
  #if(length(imputed.value)>1){imputed.value <- sample(imputed.value,1)}
  return(imputed.value)  
}

#############################
## Evaluate the imputation ##
#############################
## revise this function on 4/23/2013
Evaluate <- function(imputed.value,true.value,miss.type,col,Complete){
  if (miss.type=="con"){
    ## normalized square error
    ## could be >1
    accu.v <-((true.value-imputed.value)/sd(na.omit(Complete[,col])))^2
  }
  else if (miss.type=="nom"){
    ## 0~1
    if(true.value==imputed.value){accu.v <- 0}
    else {accu.v <- 1}
  }
  else {
    ## 0~1
    accu.v <- ((true.value-imputed.value)/(nlevels(as.factor(Complete[,col]))-1))^2}
  accu.v  
}

#################################
## Function of Mean Imputation ##
#################################
## revise this function on 4/23/2013
each.mv.m.i <- function(col,mv.comp,Type){
  dat <- as.numeric(na.omit(mv.comp[,col]))
  if (Type[col]=="con"){imputed.m.i <- mean(dat)}
  else if (Type[col]=="nom"){
    imputed.m.i <- as.numeric(names(which.max(table(dat))))}
  else {imputed.m.i <- round(mean(dat))
        if(imputed.m.i>max(dat)){imputed.m.i=max(imputed.m.i)}}
  return(imputed.m.i)  
}


##################################################################################################################
####### Function Get imputed value(single missing single neighbor if there is a neighbor) KNN.v####
##################################################################################################################
Imp.Single <- function(neighbor.all,regression.all,miss.col,miss.row,neighbor.col,Type,mv.data){
  
  coeff.neighbor <- which(neighbor.all==neighbor.col)  ## the regression coefficient corresponding to each neighbor
  coeff <- regression.all[[coeff.neighbor]]
  ## Each pair of missing value and neighbor
  dat <- mv.data[,c(miss.col,neighbor.col)]
  depend <- dat[miss.row,2]
  dat <- na.omit(dat)
  min.1 <- min(dat[,1])
  max.1 <- max(dat[,1])
  min.2 <- min(dat[,2])
  max.2 <- max(dat[,2])
  level.1 <- nlevels(as.factor(dat[,1]))
  level.2 <- nlevels(as.factor(dat[,2]))
  type <- c(Type[c(miss.col,neighbor.col)]) ## variable type for missing value and neighbor
  if(sum(type==c("con","con"))==2){
    imp <- coeff[1]+coeff[2]*depend
  }
  if(sum(type==c("con","nom"))==2){
    possible <- sort(unique(dat[,2]))
    if(depend%in%possible==FALSE){depend <- sample(as.numeric(row.names(table(dat[,2]))),1,prob=table(dat[,2])/sum(table(dat[,2])))}
    no <- which(possible==depend)
    if(no==1){imp <- coeff[1]}
    else{
      imp <- coeff[1] + coeff[no] * 1}
  }
  if(sum(type==c("con","ord"))==2){
    imp <- coeff[1]+coeff[2]*depend
  }
  if(sum(type==c("ord","con"))==2){
    #depend <- depend - mean(dat[,2])  
    ## if Sparse cell when high missing,we cannot regress. Use other methods to impute in this case #
    if(unique(is.na(coeff))==TRUE){obs <- which(abs(depend-dat[,2])==min(abs(depend-dat[,2]))) ## similiar to KNN.s
                                   imp <- dat[obs,1]}
    else{
      cut <- coeff[[2]]
      predictor <- coeff[[1]]
      p.value <- vector(length=level.1)
      p.value[1] <- 1/(1+exp(-(cut[1]-predictor*depend))) 
      for (i in 2:(level.1-1)){
        p.value[i] <- 1/(1+exp(-(cut[i]-predictor*depend))) - 1/(1+exp(-(cut[i-1]-predictor*depend)))
      }
      p.value[level.1] <- 1 - 1/(1+exp(-(cut[i]-predictor*depend)))
      possible.value <- c(min.1:(min.1+level.1-1))
      imp <- max(possible.value[p.value==max(p.value)])
    }  
  }
  if(sum(type==c("ord","ord"))==2){
    possible <- sort(unique(dat[,2]))
    if(min(table(dat[,1],dat[,2]))<10){ ## if sparse cell use conditional probability(use contingency table)
      if(depend%in%possible==FALSE){depend <- sample(as.numeric(row.names(table(dat[,2]))),1,prob=table(dat[,2])/sum(table(dat[,2])))}
      table <- table(dat[,1],dat[,2])
      if(depend - min.2 + 1>ncol(table)){depend <- ncol(table)}
      else{miss.dist <- table[,depend - min.2 + 1]}
      name <- as.numeric(names(miss.dist[which(miss.dist>0)]))
      prob <- miss.dist[which(miss.dist>0)]/sum(miss.dist)   
      if(length(name)==1){imp <- name}
      else{imp <- max(name[which(prob==max(prob))])}} ## similiar to mean imputation    
    else{
      cut <- coeff[[2]]
      predictor <- coeff[[1]]
      p.value <- vector(length=level.1)
      p.value[1] <- 1/(1+exp(-(cut[1]-predictor*depend))) 
      for (i in 2:(level.1-1)){
        p.value[i] <- 1/(1+exp(-(cut[i]-predictor*depend))) - 1/(1+exp(-(cut[i-1]-predictor*depend)))
      }
      p.value[level.1] <- 1 - 1/(1+exp(-(cut[i]-predictor*depend)))
      possible.value <- c(min.1:(min.1+level.1-1))
      imp <- max(possible.value[p.value==max(p.value)])
    }}
  if(sum(type==c("ord","nom"))==2){
    if(min(table(dat[,1],dat[,2]))<10){ ## if sparse cell use conditional probability(use contingency table)
      table <- table(dat[,1],dat[,2])
      possible <- sort(unique(dat[,2]))
      if(depend%in%possible==FALSE){depend <- sample(as.numeric(row.names(table(dat[,2]))),1,prob=table(dat[,2])/sum(table(dat[,2])))}
      no <- which(possible==depend)
      miss.dist <- table[,no]
      name <- as.numeric(names(miss.dist[which(miss.dist>0)]))
      prob <- miss.dist[which(miss.dist>0)]/sum(miss.dist)
      if(length(name)==1){imp <- name}
      else{imp <- max(name[prob==max(prob)])}
    }
    else{
      cut <- coeff[[2]]
      predictor <- coeff[[1]]
      possible <- sort(unique(dat[,2]))
      no <- which(possible==depend)
      if(depend == possible[1]){predictor.use <- 0}
      else{predictor.use <- predictor[no]}
      p.value <- vector(length=length(possible))
      p.value[1] <- 1/(1+exp(-(cut[1]-predictor.use))) 
      for (i in 2:(level.1-1)){
        p.value[i] <- 1/(1+exp(-(cut[i]-predictor.use))) - 1/(1+exp(-(cut[i-1]-predictor.use)))
      }
      p.value[level.1] <- 1 - 1/(1+exp(-(cut[i]-predictor.use)))
      imp <- max(possible[p.value==max(p.value)])    	
    }
  }
  if(sum(type==c("nom","con"))==2){ 
    if(level.1==2){ ##logistic regression
      if(min(table(dat[,1]))<10){imp <- sample(as.numeric(row.names(table(dat[,1]))),1,prob=table(dat[,1])/sum(table(dat[,1])))}
      else{ 
        p.i <- exp(coeff[1]+coeff[2]*depend)/(1+exp(coeff[1]+coeff[2]*depend))
        if(is.na(p.i)==TRUE){p.i <- 1}
        if(p.i > 0.5){imp <- max.1} ## P(outcome=1) > P(outcome=0)
        else {imp <- min.1}}}
    else if(level.1 > 2) { ##multinomial logistic regression
      if(min(table(dat[,1]))<10){imp <- sample(as.numeric(row.names(table(dat[,1]))),1,prob=table(dat[,1])/sum(table(dat[,1])))}
      else{
        logit <- vector(length=nlevels(as.factor(dat[,1]))-1) 
        for (i in 1:(nlevels(as.factor(dat[,1]))-1)){
          logit[i] <- coeff[i] + depend * coeff[i+level.1-1]
        }
        exp.beta <- exp(logit)
        p <- vector(length=nlevels(as.factor(dat[,1])))
        p[1] <- 1/(1+sum(exp.beta))
        for (i in 2:nlevels(as.factor(dat[,1]))){
          p[i] <- exp.beta[i-1]/(1+sum(exp.beta))}
        value <- c(min.1:max.1)
        imp <- max(value[p==max(p)])}
    }
  }
  if(sum(type==c("nom","ord"))==2){
    if(level.1==2){
      if(min(table(dat[,1],dat[,2]))<10){table <- table(dat[,1],dat[,2])
                                         possible <- sort(unique(dat[,2]))
                                         if(depend%in%possible==FALSE){depend <- sample(as.numeric(row.names(table(dat[,2]))),1,prob=table(dat[,2])/sum(table(dat[,2])))}     
                                         no <- which(possible==depend)
                                         miss.dist <- table[,no]   
                                         name <- as.numeric(names(miss.dist[miss.dist>0]))
                                         prob <- miss.dist[miss.dist>0]/sum(miss.dist)
                                         if(length(name)==1){imp <- name}
                                         else{imp <- max(name[prob==max(prob)])}
      }
      else{
        possible <- sort(unique(dat[,2]))
        if(depend%in%possible==FALSE){depend <- sample(as.numeric(row.names(table(dat[,2]))),1,prob=table(dat[,2])/sum(table(dat[,2])))}     
        if(depend==min.2){p.i <- exp(coeff[1])/(1+exp(coeff[1]))}
        else{p.i <- exp(coeff[1]+coeff[depend-min.2+1]*1)/(1+exp(coeff[1]+coeff[depend-min.2+1]*1))}	
        if(is.na(p.i)==TRUE){p.i <- 1}
        if(p.i > 0.5){imp <- max.1} ## P(outcome=1) > P(outcome=0)
        else {imp <- min.1}
      }
    }
    else if(level.1 > 2) { ##multinomial logistic regression
      if(min(table(dat[,1],dat[,2]))<10){table <- table(dat[,1],dat[,2])
                                         possible <- sort(unique(dat[,2]))
                                         if(depend%in%possible==FALSE){depend <- sample(as.numeric(row.names(table(dat[,2]))),1,prob=table(dat[,2])/sum(table(dat[,2])))}     
                                         no <- which(possible==depend)
                                         miss.dist <- table[,no]   
                                         name <- as.numeric(names(miss.dist[miss.dist>0]))
                                         prob <- miss.dist[miss.dist>0]/sum(miss.dist)
                                         if(length(name)==1){imp <- name}
                                         else{imp <- max(name[prob==max(prob)])}
      }
      else{
        logit <- vector(length=nlevels(as.factor(dat[,1]))-1) ##CALCULATE BETA
        for (i in 1:(nlevels(as.factor(dat[,1]))-1)){
          logit[i] <- coeff[i] + depend * coeff[i+level.1-1]
        }
        exp.beta <- exp(logit)
        p <- vector(length=nlevels(as.factor(dat[,1])))
        p[1] <- 1/(1+sum(exp.beta))
        for (i in 2:nlevels(as.factor(dat[,1]))){
          p[i] <- exp.beta[i-1]/(1+sum(exp.beta))}
        value <- c(min.1:max.1)
        imp <- max(value[p==max(p)])}
    }
  }
  if(sum(type==c("nom","nom"))==2){
    if(level.1==2){ ##logistic regression
      if(min(table(dat[,1],dat[,2]))<10){table <- table(dat[,1],dat[,2])
                                         possible <- sort(unique(dat[,2]))
                                         if(depend%in%possible==FALSE){depend <- sample(as.numeric(row.names(table(dat[,2]))),1,prob=table(dat[,2])/sum(table(dat[,2])))}     
                                         no <- which(possible==depend)
                                         miss.dist <- table[,no]   
                                         name <- as.numeric(names(miss.dist[miss.dist>0]))
                                         prob <- miss.dist[miss.dist>0]/sum(miss.dist)
                                         if(length(name)==1){imp <- name}
                                         else{imp <- max(name[prob==max(prob)])}
      }
      else{        
        possible <- sort(unique(dat[,2]))
        if(depend%in%possible==FALSE){depend <- sample(as.numeric(row.names(table(dat[,2]))),1,prob=table(dat[,2])/sum(table(dat[,2])))}     
        no <- which(possible==depend)
        if(depend>min.2){p.i <- exp(coeff[1]+coeff[no])/(1+exp(coeff[1]+coeff[no]))}	
        else{p.i <- exp(coeff[1])/(1+exp(coeff[1]))}
        if(is.na(p.i)==TRUE){p.i <- 1}
        if(p.i > 0.5){imp <- max.1} ## P(outcome=1) > P(outcome=0)
        else {imp <- min.1}
      }
    }
    else if(level.1 > 2) { ##multinomial logistic regression
      if(min(table(dat[,1],dat[,2]))<10){table <- table(dat[,1],dat[,2])
                                         possible <- sort(unique(dat[,2]))
                                         if(depend%in%possible==FALSE){depend <- sample(as.numeric(row.names(table(dat[,2]))),1,prob=table(dat[,2])/sum(table(dat[,2])))}     
                                         no <- which(possible==depend)
                                         miss.dist <- table[,no]   
                                         name <- as.numeric(names(miss.dist[miss.dist>0]))
                                         prob <- miss.dist[miss.dist>0]/sum(miss.dist)
                                         if(length(name)==1){imp <- name}
                                         else{imp <- max(name[prob==max(prob)])}
      }
      else{
        possible <- sort(unique(dat[,2]))
        if(depend%in%possible==FALSE){depend <- sample(as.numeric(row.names(table(dat[,2]))),1,prob=table(dat[,2])/sum(table(dat[,2])))}     
        no <- which(possible==depend)
        logit <- vector(length=nlevels(as.factor(dat[,1]))-1) 
        for (i in 1:(nlevels(as.factor(dat[,1]))-1)){
          if(no==1) logit[i]<-coeff[i]
          else logit[i] <- coeff[i] + coeff[(no-1)*(level.1-1)+i]
        }
        exp.beta <- exp(logit)
        p <- vector(length=nlevels(as.factor(dat[,1])))
        p[1] <- 1/(1+sum(exp.beta))
        for (i in 2:nlevels(as.factor(dat[,1]))){
          p[i] <- exp.beta[i-1]/(1+sum(exp.beta))}
        value <- c(min.1:max.1)
        imp <- max(value[p==max(p)])}
    }
  }
  if(length(imp)>1){imp <- sample(imp,1)}
  imp
}
##############################################
####  Get coefficient for each possible reg ##
##############################################
cal.reg.coef <- function(mv.data,miss.col,neighbor.col,Type){
  dat <- na.omit(mv.data[,c(miss.col,neighbor.col)])
  table <- table(dat[,1],dat[,2])
  m.type <- Type[miss.col]
  n.type <- Type[neighbor.col]
  if("con" %in% m.type){ ## linear regression
    if("con" %in% n.type){
      coeff <- as.vector(lm(dat[,1]~dat[,2])$coefficients) 
    } else if("ord" %in% n.type) ## treat ordinal variable as continuous
    {coeff <- as.vector(lm(dat[,1]~dat[,2])$coefficients)
    } else if("nom" %in% n.type)
    {min <- min(dat[,2],na.rm=TRUE)
     max <- max(dat[,2],na.rm=TRUE)
     dat=data.frame(dat)
     dat[,2] <- factor(dat[,2])  ## Create dummy variables
     coeff <- as.vector(lm(dat[,1]~dat[,2])$coefficients)    
    }}
  else if("ord" %in% m.type){ ## ordinal logistic regression
    min.1 <- min(dat[,1])
    max.1 <- max(dat[,1])
    min.2 <- min(dat[,2])
    max.2 <- max(dat[,2])
    level.1 <- nlevels(as.factor(dat[,1]))
    level.2 <- nlevels(as.factor(dat[,2]))
    table <- table(dat[,1],dat[,2])
    if("con" %in% n.type){ ## beta are all same, intercept different
      if((min(table(dat[,1]))<10)|(level.1-(max.1-min.1+1)!=0)){coeff <- NA}
      else{
        #dat[,2] <- dat[,2] - mean(dat[,2])
        t <- try(polr(as.ordered(dat[,1])~dat[,2],data=dat),T)
        if(is.character(t[[1]])!=TRUE){         
          polr.out <- polr(as.ordered(dat[,1])~dat[,2],data=dat)
          predictor <- polr.out$coefficients
          cut <- polr.out$zeta
          coeff <- list(predictor,cut)}
        else{coeff <- NA}}  ## this is a list
    } else if("ord" %in% n.type){  ## ordinal logistic regression or linear regression
      if(min(table)<10){coeff <- NA}
      else{
        polr.out <- polr(as.ordered(dat[,1])~dat[,2],data=dat)
        predictor <- polr.out$coefficients
        min <- min(dat[,1])
        cut <- polr.out$zeta
        coeff <- list(predictor,cut)
      }
    }
    else{
      ## use ordinal logistic regression create dummy variables(as.factor)
      if(min(table)<10){coeff <- NA}
      else{
        dat=data.frame(dat)
        dat[,2] <- as.factor(dat[,2]) 
        level.1 <- nlevels(as.factor(dat[,1]))
        polr.out <- polr(as.ordered(dat[,1])~dat[,2],data=dat)
        predictor <- polr.out$coefficients
        cut <- polr.out$zeta
        coeff <- list(predictor,cut)
      }
    }
  }
  else if("nom" %in% m.type){  ##logistic regression
    min.1 <- min(dat[,1])
    max.1 <- max(dat[,1])
    min.2 <- min(dat[,2])
    max.2 <- max(dat[,2])
    level.1 <- nlevels(as.factor(dat[,1]))
    level.2 <- nlevels(as.factor(dat[,2]))
    table <- table(dat[,1],dat[,2])
    if(n.type %in% c("con")){
      if(min(table(dat[,1]))<10){coeff <- NA}
      else{
        if(level.1==2){
          if(min(dat[,1])>0){dat[,1] <- dat[,1]-min.1}  ## change into 0 and 1 format
          mylogit<- glm(dat[,1]~dat[,2], family=binomial(link="logit"), na.action=na.pass)
          coeff <- mylogit$coefficients
        }
        else if(level.1 > 2) { ##multinomial logistic regression
          colnames(dat) <- c("missing","depend")
          row.names(dat) <- NULL
          dat=data.frame(dat)
          dat[,1]=as.factor(dat[,1])
          a <- mlogit.data(dat, varying=NULL, choice="missing", shape="wide")
          mlogit.model<- mlogit::mlogit(missing~1|depend,data=a,reflevel=1)  
          coeff <- as.matrix(mlogit.model$coefficient)
        }
      }
    }
    else if("ord" %in% n.type){
      if(level.1==2){      
        if(min(table)<10){coeff <- NA}
        else{
          if(min.1>0){dat[,1] <- dat[,1]-min.1}  ## change into 0 and 1 format
          mylogit<- glm(dat[,1]~dat[,2], family=binomial(link="logit"), na.action=na.pass)
          coeff <- mylogit$coefficients
        }
      }
      else if(level.1 > 2) { ##multinomial logistic regression
        if(min(table)<10){coeff <- NA}
        else{
          colnames(dat) <- c("missing","depend")
          row.names(dat) <- NULL
          dat=data.frame(dat)
          dat[,1]=as.factor(dat[,1])
          a <- mlogit.data(dat, varying=NULL, choice="missing", shape="wide")
          mlogit.model<- mlogit::mlogit(missing~1|depend,data=a,reflevel=1)  ##
          coeff <- as.matrix(mlogit.model$coefficient)
        }
      }
    }
    else if("nom" %in% n.type){## no sparse cell
      dat[,2] <- as.factor(dat[,2])         
      if(level.1==2){ ##logistic regression
        if(min(table)<10){coeff <- NA}
        else{
          if(min.1>0){dat[,1] <- dat[,1]-min.1}  ## change into 0 and 1 format
          mylogit<- glm(dat[,1]~dat[,2], family=binomial(link="logit"), na.action=na.pass)
          coeff <- mylogit$coefficients
        }
      }
      else if(level.1 > 2) { ##multinomial logistic regression
        if(min(table)<10){coeff <- NA}
        else{
          colnames(dat) <- c("missing","depend_value")
          row.names(dat) <- NULL
          dat=data.frame(dat)
          dat[,1]=as.factor(dat[,1])
          dat[,2]=as.factor(dat[,2])
          a <- mlogit.data(dat, varying=NULL, choice="missing", shape="wide")
          mlogit.model<- mlogit::mlogit(missing~1|depend_value,data=a,reflevel=1)  
          coeff <- as.matrix(mlogit.model$coefficient)  
        }
      }
    }
  }
  return(coeff)
}


### revise on 4/23/2013
take.mean <- function(result){
  l <- length(result)  ## result should be a list
  p <- length(result[[1]])
  mean.output = unlist(lapply(1:p,function(x) mean(unlist(lapply(1:l,function(y) result[[y]][x])))))
  mean.output
}

loc.gen.missing <- function(mv.no,Type,nr,nc){
  col <- floor(mv.no/nr)+1
  row <- mv.no-nr*floor(mv.no/nr)
  if(row==0){(row=nr) & (col=col-1)}
  miss.type <- Type[col]  # type of missing value
  return(c(row,col,miss.type))
}

Summary.Result <- function(Error,miss.type){
  result.con <- sqrt(mean(Error[miss.type=="con"]))
  result.nom <- table(Error[miss.type=="nom"])[row.names(as.matrix(table(Error[miss.type=="nom"])))==1]/sum(table(Error[miss.type=="nom"]))
  result.ord <- sqrt(mean(Error[miss.type=="ord"]))  
  return(c(result.con,result.nom,result.ord))
}

round.p <- function(x){
  if(x<0){x = 0}
  else if(x>1){x=1}
  else{x=x}
  x
}


First.missing.simulation <- function(ind){
  nc <- ncol(Complete)
  nr <- nrow(Complete)
  Missing <- round(perc*nc*nr) 
  mv.comp <- Complete
  mv.no = missing.data[[ind]]
  
  ## position of missing
  missing.info = t(sapply(mv.no,function(x) loc.gen.missing(x,Type,nr,nc)))
  col <- as.numeric(missing.info[,2])
  row <- as.numeric(missing.info[,1])
  miss.type <- missing.info[,3]  
  
  ## generate missingness   
  for (j in 1:Missing)
  {
    mv.comp[row[j],col[j]] <- NA
  }
  dist.1 <- mixedDist(mv.comp,Type)
  print("dist")
  dist.var <- dist.1 + t(dist.1)
  dist.var <- as.matrix(dist.var)
  mv.comp.1 <- as.data.frame(mv.comp)
  for(i in 1:length(which(Type=="con"))){
    mv.comp.1[,which(Type=="con")[i]] <- as.numeric(mv.comp[,which(Type=="con")[i]])}
  for(i in 1:length(which(Type=="ord"))){
    mv.comp.1[,which(Type=="ord")[i]] <- as.ordered(mv.comp[,which(Type=="ord")[i]])}
  for(i in 1:length(which(Type=="nom"))){
    mv.comp.1[,which(Type=="nom")[i]] <- as.factor(mv.comp[,which(Type=="nom")[i]])}
  ## calculate each imputed value of each missingness
  dist.sub <- as.matrix(daisy(mv.comp.1,metric="gower"))
  col.i.5 <- vector("list",length=Missing)
  col.i.10 <- vector("list",length=Missing)
  col.i.15 <- vector("list",length=Missing)
  
  colnames(dist.var) <- colnames(Complete)
  row.names(dist.var) <- colnames(Complete)
  true.value <- vector(length=Missing)
  w.1.5 <- vector("list",length=Missing)
  w.1.10 <- vector("list",length=Missing)
  w.1.15 <- vector("list",length=Missing)
  
  r.1 = vector("list",length=Missing)
  
  ## prepare for KNN.V
  ## Get col.i true.value
  for (e in  1:Missing){
    ## mv.no: from 1 to perc * row *col
    ## neighbor.no: from 1 to k 
    true.value[e] <- Complete[row[e],col[e]] 
    candidate <- dist.var[col[e],-unique(c(which(is.na(mv.comp[row[e],])==TRUE),which(is.na(dist.var[col[e],])==TRUE)))] 
    index=order(candidate)[1:k]
    neighbor <- candidate[index]
    
        
    col.i.15[[e]] <- which(colnames(mv.comp)%in%row.names(as.matrix(neighbor)))
    col.i.5[[e]] <- which(colnames(mv.comp)%in%row.names(as.matrix(neighbor))[1:5])
    col.i.10[[e]] <- which(colnames(mv.comp)%in%row.names(as.matrix(neighbor))[1:10])
    w.1.15[[e]] <- (1-dist.var[col[e],col.i.15[[e]]])^2/(1-(1-dist.var[col[e],col.i.15[[e]]])^2+10^(-6))
    
    w.1.5[[e]] <- (1-dist.var[col[e],col.i.5[[e]]])^2/(1-(1-dist.var[col[e],col.i.5[[e]]])^2+10^(-6))
    w.1.10[[e]] <- (1-dist.var[col[e],col.i.10[[e]]])^2/(1-(1-dist.var[col[e],col.i.10[[e]]])^2+10^(-6))
  }
  #########################################################################
  ###### Get possible regression coefficient needed and mean imputation ####
  #########################################################################
  regression.r <- vector("list",length=ncol(Complete))
  neighbor.r <- vector("list",length=ncol(Complete))
  mean.imp <- vector(length=ncol(Complete))
  for (i in 1:ncol(Complete)){
    mv.id <- which(col==i)
    if(length(mv.id)==0){neighbor.r[[i]] <- NA
    }else if(length(mv.id)==1){neighbor.r[[i]] <- col.i.15[[mv.id[1]]]}
    else{
      neighbor.u <- col.i.15[[mv.id[1]]]
      for (h in 1:(length(mv.id)-1)){
        neighbor.u <- unique(c(neighbor.u,col.i.15[[mv.id[h+1]]]))
      }
      neighbor.r[[i]] <- neighbor.u
    }
    if (length(na.omit(neighbor.r[[i]]))>0){
      n.n <- length(neighbor.r[[i]])
      regression.each <- vector("list",length=n.n) ##k:no of neighbor
      for (u in 1:n.n){
        regression.each[[u]] <- cal.reg.coef(mv.comp,i,neighbor.r[[i]][u],Type)
      }
      regression.r[[i]] <- regression.each}
    else{
      regression.r[[i]] =NA
    }
    mean.imp[i] <- each.mv.m.i(i,mv.comp,Type)    
  }  
  imp.15 <- vector("list",length=Missing)
  imp.10 <- vector("list",length=Missing)
  imp.5 <- vector("list",length=Missing)
  
    Calculate.imp <- function(i){
      print(i)
      temp <- vector(length=length(col.i.15[[i]]))
      if(length(col.i.15[[i]])>0){
        temp <- sapply(1:length(col.i.15[[i]]),function(x) Imp.Single(neighbor.r[[col[i]]],regression.r[[col[i]]],col[i],row[i],col.i.15[[i]][x],Type,mv.comp))
      }
      else{temp <- sample(na.omit(mv.comp[,col[i]]),k)
      }
      imp.15[[i]] <- na.omit(temp)
      if(length(which(is.na(temp)==TRUE))==0){w.1.15[[i]] <- w.1.15[[i]]                                           
      }
      else{w.1.15[[i]] <- w.1.15[[i]][-which(is.na(temp)==TRUE)]
      }
      ##
      if(length(col.i.10[[i]])>0){
        temp <- imp.15[[i]][which(col.i.15[[i]]%in%col.i.10[[i]])]
      }
      else{temp <- sample(na.omit(mv.comp[,col[i]]),10)
      }
      imp.10[[i]] <- na.omit(temp)
      if(length(which(is.na(temp)==TRUE))==0){w.1.10[[i]] <- w.1.10[[i]]                                           
      }
      else{w.1.10[[i]] <- w.1.10[[i]][-which(is.na(temp)==TRUE)]
      }
      ##
      if(length(col.i.5[[i]])>0){
        temp <- imp.15[[i]][which(col.i.15[[i]]%in%col.i.5[[i]])]
      }
      else{temp <- sample(na.omit(mv.comp[,col[i]]),5)
      }
      imp.5[[i]] <- na.omit(temp)
      if(length(which(is.na(temp)==TRUE))==0){w.1.5[[i]] <- w.1.5[[i]]                                           
      }
      else{w.1.5[[i]] <- w.1.5[[i]][-which(is.na(temp)==TRUE)]
      }
      return(list(imp.5[[i]],w.1.5[[i]],imp.10[[i]],w.1.10[[i]],imp.15[[i]],w.1.15[[i]]))
  }
  imputed.all = lapply(1:Missing,Calculate.imp)
  
  k.imputed.value.1.5 <- imputed.all
  k.imputed.value.1.10 <- imp.10
  k.imputed.value.1.15 <- imp.15
  
  final.var.5 <- vector(length=Missing)
  final.var.10 <- vector(length=Missing)
  final.var.15 <- vector(length=Missing)
  
  accu.v.5 <- vector(length=Missing)
  accu.v.10 <- vector(length=Missing)
  accu.v.15 <- vector(length=Missing)
  
  Get.final.value <- function(x){
    print(x)
    if(length(imputed.all[[x]][[2]])==0){imputed.all[[x]][[2]] <- rep(1,5)}
    if(length(imputed.all[[x]][[4]])==0){imputed.all[[x]][[4]] <- rep(1,10)}
    if(length(imputed.all[[x]][[6]])==0){imputed.all[[x]][[6]] <- rep(1,15)}
    
    final.var.5 <- determin.i(imputed.all[[x]][[1]],imputed.all[[x]][[2]],miss.type[x],1)
    final.var.10 <- determin.i(imputed.all[[x]][[3]],imputed.all[[x]][[4]],miss.type[x],1)
    final.var.15 <- determin.i(imputed.all[[x]][[5]],imputed.all[[x]][[6]],miss.type[x],1)
    
    accu.v.5 <- Evaluate(final.var.5,true.value[x],miss.type[x],col[x],Complete)
    accu.v.10 <- Evaluate(final.var.10,true.value[x],miss.type[x],col[x],Complete)
    accu.v.15 <- Evaluate(final.var.15,true.value[x],miss.type[x],col[x],Complete)
    return(c(accu.v.5,accu.v.10,accu.v.15))
  }
    result.knn.v = t(sapply(1:Missing,function(x) Get.final.value(x)))
  
  print("Imp.Var finish")
  
  
  #####################################################
  ## KNN.s: Using k nearest subject do MV imputation ##
  #####################################################
  ## calculate the accuracy of imputation
  ## calculate the accuracy of random imputation
  ## calculate the accuracy of mean/majority vote imputation
  final.sub.5 <- vector(length=Missing)
  final.sub.10 <- vector(length=Missing)
  final.sub.15 <- vector(length=Missing)
  accu.s.5 <- vector(length=Missing)
  accu.s.10 <- vector(length=Missing)
  accu.s.15 <- vector(length=Missing)
  
  k.imputed.value.2 <- vector("list",length=Missing)
  w.2.5 <- vector("list",length=Missing)
  w.2.10 <- vector("list",length=Missing)
  w.2.15 <- vector("list",length=Missing)
  
  for (i in 1:Missing){
    candidate <- dist.sub[-c(which(is.na(mv.comp.1[,col[i]])==TRUE),row[i]),row[i]]
    dist <- candidate[order(candidate)][1:k]
    k.imputed.value.2[[i]] <- mv.comp[sapply(row.names(as.matrix(dist)),function(x) which(row.names(mv.comp.1)%in%x)),col[i]]
    row.i <- which(row.names(mv.comp.1)%in%row.names(as.matrix(dist)))
    w.2.15[[i]] <- (1- dist.sub[row[i],row.i[1:15]])^2/(1-(1- dist.sub[row[i],row.i[1:15]])^2+10^(-6))
    final.sub.15[i] <- determin.i(k.imputed.value.2[[i]][1:15],w.2.15[[i]],miss.type[i],1)
    dist <- candidate[order(candidate)][1:10]
    k.imputed.value.2[[i]] <- mv.comp[sapply(row.names(as.matrix(dist)),function(x) which(row.names(mv.comp.1)%in%x)),col[i]]
    row.i <- which(row.names(mv.comp.1)%in%row.names(as.matrix(dist)))
    w.2.10[[i]] <- (1- dist.sub[row[i],row.i[1:10]])^2/(1-(1- dist.sub[row[i],row.i[1:10]])^2+10^(-6))
    final.sub.10[i] <- determin.i(k.imputed.value.2[[i]][1:10],w.2.10[[i]],miss.type[i],1)
    dist <- candidate[order(candidate)][1:5]
    k.imputed.value.2[[i]] <- mv.comp[sapply(row.names(as.matrix(dist)),function(x) which(row.names(mv.comp.1)%in%x)),col[i]]
    row.i <- which(row.names(mv.comp.1)%in%row.names(as.matrix(dist)))
    w.2.5[[i]] <- (1- dist.sub[row[i],row.i[1:5]])^2/(1-(1- dist.sub[row[i],row.i[1:5]])^2+10^(-6))
    final.sub.5[i] <- determin.i(k.imputed.value.2[[i]][1:5],w.2.5[[i]],miss.type[i],1)
    
    ## calculate the random imputation value
    accu.s.5[i] <- Evaluate(final.sub.5[i],true.value[i],miss.type[i],col[i],Complete)
    accu.s.10[i] <- Evaluate(final.sub.10[i],true.value[i],miss.type[i],col[i],Complete)
    accu.s.15[i] <- Evaluate(final.sub.15[i],true.value[i],miss.type[i],col[i],Complete)
    
  }
  
  result = rbind(Summary.Result(result.knn.v[,1],miss.type),Summary.Result(result.knn.v[,2],miss.type),Summary.Result(result.knn.v[,3],miss.type),Summary.Result(accu.s.5,miss.type),Summary.Result(accu.s.10,miss.type),Summary.Result(accu.s.15,miss.type))    
  row.names(result) = c("KNN-V(K=5)","KNN-V(K=10)","KNN-V(K=15)","KNN-S(K=5)","KNN-S(K=10)","KNN-S(K=15)")    
  colnames(result) = c("con","nom","ord")
  return(result)
}
